/*A*//* Head  */
/****h*  makepattern.c *********
 *
 *	NAME
 *   makepattern.c
 *
 *	FUNCTION
 *    This part generates the pattern from the
 *    siteswap.
 *    Maybe you have to change something here to
 *    get the required output, but look first at
 *    output.c
 *
 *	CREATION DATE
 *	  4-Mar-1997
 *
 *  HISTORY
 *    040397 disconected this part from main.c
 *    150397 simple pattern work
 *    160397 the period of the pattern is now calculated here
 *    310397 multiplex seems to work
 *    010597 fixed multiplex problem [123]
 *
 *  TODO
 *
 *  BUGS 
 *
 *  BEAST METHODS
 *
 *    makepattern()
 *
                  *******
*/
/*E*/

/*-----------------------------------------*/

/*A*//*  External routines and Structures */

#include "define.h"

/* {{{ */
/* for folding recognition */
/* }}} */


/*E*/

/*-----------------------------------------*/

/*A*/ /*  Global variables */

extern struct OPTIONS options;	/*  for some options */
extern int *siteswap;		/* external pointer for the decoded inputstring */
extern int site1;
extern int site2;
int **pattern;			/* pointer to array of pointers, which are pointing to an int array */
int patt1;			/* multiplier for array access */
int *objects;			/* array for object data */
int obje1;			/* for access */
/*
   small global arrays (not dynamic allocated)
 */

extern int hdelay[];		/* external table for handdelay */

/*E*/

/*-----------------------------------------*/

/*A*/ int makepattern () {
/****** makepattern.c/makepattern *********
 *
 *	NAME
 *	  makepattern
 *
 *	FUNCTION
 *    This part generates the pattern from the siteswap.
 *
 *  SYNOPSIS
 *    stderror = makepattern()
 *
 *  RESULT
 *    stderror - zero for success, else an error code is returned
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *
 *******
 */

  int count1;			/* counter */
  int th;			/* ThrowHeight */
  int tt;			/* ThrowTarget */
  int targetpos;		/* Targetposition in time */
  int targetpos1;		/* Targetposition in array */

  int hand;			/* 0=left 1=right */
  int targethand;		/* targethand encoded with person */
  int person;			/* 0=first 1=second person */
  int phand;			/* adress in the pattern array for hand+person */
  int tperson;			/* targetperson after throw */
  int pos;			/* position in patternarray */
  int spos;			/* position in siteswap */
  int throwobject;		/* object which was thrown */
  int tmplex;			/* target multiplex position */
  int mplex;			/* multiplex loop just the counter */
  int mplex1;			/* multiplex loop the real position */

  int usedobj;			/* the object which was set last */
  int occurobj;			/* order of occurence */
  int exit;			/* shows the loop to exit */

  int allperiods;		/* true if all periods have been calculated */
  int period1;			/* normal period of pattern */
//int period2;                          /* wrap 1 period of pattern */
  //int period3;                          /* wrap 2 period of pattern */

  if (options.debug)
    printf ("makepattern started\n\n");

  /* get memory for the patternpointers */

  pattern = (int **) calloc (sizeof (int *), MAXOUTLINES);

  if (pattern == NULL)
    {
      printf ("Error: Not enough memory for pattern 1");
      return 1;
    }
  patt1 = options.mplexmax + 1;

  /* first get minimum memory (MAXTHROW) */


  for (count1 = 0; count1 < MAXTHROW; count1++)
    {
      pattern[count1] = (int *) calloc (sizeof (int), (options.people) * (2) * (options.mplexmax + 1));
      if (pattern[count1] == NULL)
        {
          printf ("Error: Not enough memory for pattern 2");
          return 1;
        }
    }

  /* get memory for the objects */

  objects = (int *) calloc (sizeof (int), (options.objects) * 10);

  if (objects == NULL)
    {
      printf ("Error: Not enough memory for objects");
      return 1;
    }
  obje1 = 10;
  usedobj = 0;			/* none for now */
  occurobj = 0;
  pos = 0;
  exit = FALSE;
  period1 = 2;

  do
    {				/*      go thru the pattern until it is ready */
      for (hand = 0; hand < 2; hand++)
        {
          for (person = 0; person < (options.people); person++)
            {
              phand = person * 2 + hand;	/* adress in the pattern array for hand+person */
              spos = (pos * 2 + hand) % (options.length);	/* position in the siteswap string */
              /* check if we have to set a new object */
              if (usedobj < (options.objects))
                {
/** set a new object **/
                  for (mplex = 0; mplex < *(siteswap + (spos * site1) + (person * site2) + (0)); mplex++)
                    {		/* multiplex */
//printf("mplex: %d\n",mplex);
                      //printf("phand: %d\n",phand);
                      //printf("patt:%d\n",(pattern[pos][phand*patt1+0] )); /* number of objects */
                      if ((pattern[pos][phand * patt1 + 0]) <
                          (*(siteswap + (spos * site1) + (person * site2) + (0))))
                        {	/* already enough objects there ? */
                          mplex1 = pattern[pos][phand * patt1 + 0];	/* which mplex position */
                          (pattern[pos][phand * patt1 + 0])++;	/* add one object here */
                          if (*(siteswap + (spos * site1) + (person * site2) + (mplex1 * 3 + 1)))
                            {	/* th !=0  */
                              /* set the object here */
                              *(objects + (obje1 * usedobj) + (0)) = -1;	/* array is empty
                                                                             (set at end of first throw when
                                                                             all objects are set) */
                              if (options.latestthrow < pos * 2 + hand)
                                options.latestthrow = pos * 2 + hand;	/* calculate the latest throw of all */
                              *(pattern[pos]
                                + phand * patt1
                                + *(pattern[pos] + (phand * patt1) + (0))	/* set the object here */
                                ) = usedobj;
                              usedobj++;	/* next object */

                              if (options.debug)
                                printf ("new Object %d at %d multiplex %d\n", usedobj, spos, mplex1);
                            }
                          else
                            {	/* 0 throws don't need objects  TODO:check multiplex for zeroes */
                              (*(pattern[pos] + (phand * patt1) + (0))) = 1;	/* one NO object here */
                              (*(pattern[pos] + (phand * patt1) + (1))) = -1;	/* a NO object = -1 */
                            }
                        }	/* already enough objects here */
                    }		/* Multiplex loop */
                }		/* end of set new object */

/*--------------------------- Throw an object -------------------------*/

              for (mplex = 0; mplex < *(siteswap + (spos * site1) + (person * site2) + (0)); mplex++)
                {		/* multiplex loop */
                  th = (*(siteswap + (spos * site1) + (person * site2) + (1 + mplex * 3)));	/* height */
                  tt = (*(siteswap + (spos * site1) + (person * site2) + (3 + mplex * 3)));	/* target */
//printf("mplex: %d th %d\n",mplex,th);
                  if (th)
                    {		/* normal throw (not 0 = empty hand) */
                    /**** calculate targets ****/
                      if (tt < 2)
                        {
                          tperson = person;	/* self */
                        }
                      else
                        {
                          tperson = tt / 2 - 1;		/* targetperson  */
                          if (tperson == person)
                            {
                              printf ("Warning: absolute passtarget No. %d at %d is an self!\n",
                                      (pos * 2 + person) % (options.length * options.people), pos);
                            }
                        }
                      targethand = (tperson * 2) + ((phand + tt) % 2);
                      targetpos = (pos * 12) + hdelay[phand] + th;	/* target of throw in time */

//printf("targetperson: %d targettime: %d hand: %d \n",tperson,targetpos,targethand);
                      //printf ("tt: %d hdelay %d \n",tt,hdelay[targethand]);

                      if ((targetpos % 12) != (hdelay[targethand]))
                        {
                          printf ("Error: invalid throw (wrong targettime)\n");
                          return 2;
                        }

                      targetpos1 = (targetpos - hdelay[targethand] + hdelay[phand]) / 12;	/* target of throw in array */

//printf("targetarray: %d xx: \n",targetpos1);
                      //printf("TP: %d\n",(*(pattern[targetpos1]+(targethand*patt1)+(0))));

/***** check if throw is possible ****/

                      if (*(pattern[targetpos1] + (targethand * patt1) + (0)) ==
                          (*(siteswap + (((targetpos1 * 2 + targethand) % options.length) * site1)
                             + (person * site2) + (0)))
                          )
                        {
                          printf ("Error: Target of throw %d (",
                                  pos * 2 + hand + 1);
                          printthrow (pos, person, hand, 0);
                          printf (") from person %d to pos %d, person %d is not empty\n",
                                  person, targetpos1 * 2 + targethand % 2 + 1, tperson);
                          return 2;
                        }

                      (*(pattern[targetpos1] + (targethand * patt1) + (0)))++;	/* one more object here */

                      throwobject = pattern[pos][phand * patt1 + mplex + 1];	/* get the objectnumber */

//printf("throwobj: %d \n",throwobject);

                      tmplex = pattern[targetpos1][targethand * patt1 + 0];	/* set the object at targetpos */

                      pattern[targetpos1][targethand * patt1 + tmplex] = throwobject;

                      *(objects + (obje1 * throwobject) + (4)) = th;	/* set throwheight for multiplex */


                      /* test if period is reached */

//printf("thr %d tarh %d\n",throwobject,targethand);                                            

                      if ((!*(objects + (obje1 * throwobject) + (7))) &
                          ((*(objects + (obje1 * throwobject) + (0))) != -1)
                          )
                        {	/* only if not already set */
//printf("H");
                          if (tperson == *(objects + (obje1 * throwobject) + (0)))
                            {
//printf("e");
                              if (targethand % 2 == *(objects + (obje1 * throwobject) + (1)))
                                {
//printf("l");
                                  if ((targetpos % (options.length * 6)
                                       == (*(objects + (obje1 * throwobject) + (2))) % (options.length * 6)))
                                    {
//printf("l");
                                      if (tmplex == *(objects + (obje1 * throwobject) + (3)))
                                        {	/* check multiplex */
//printf("o");
                                          if (options.debug)
                                            {
                                              printf ("Period of object %d is %d\n", throwobject + 1, (targetpos - *(objects + (obje1 * throwobject) + (2))) / 6);
                                            }
                                          *(objects + (obje1 * throwobject) + (7)) = (targetpos - *(objects + (obje1 * throwobject) + (2))) / 6;
                                        }
                                    }
                                }
                            }
                        }	/* end of test period */
//printf("\n");
                      /* set the starting position of the objects */

                      if (usedobj == (options.objects))
                        {	/* all objects set? */

                          if (*(objects + (obje1 * throwobject) + (0)) == -1)
                            {	/* firsttarget */
//printf("huhu %d %d\n",throwobject,tperson);
                              *(objects + (obje1 * throwobject) + (0)) = tperson;
                              *(objects + (obje1 * throwobject) + (1)) = targethand % 2;
                              *(objects + (obje1 * throwobject) + (2)) = targetpos;	/* start position in 12/1 */
                              *(objects + (obje1 * throwobject) + (3)) = tmplex;	/* start multiplex */
                              *(objects + (obje1 * throwobject) + (4)) = 0;	/* starting position is defined */
                              *(objects + (obje1 * throwobject) + (5)) = 0;	/* path is not determined */
                              *(objects + (obje1 * occurobj) + (6)) = throwobject;	/* reihenfolge */
//printf("object %d occur %d \n",throwobject,occurobj);
                              occurobj++;
                              *(objects + (obje1 * throwobject) + (7)) = 0;	/* init for periodlength */
                              *(objects + (obje1 * throwobject) + (8)) = 0;
                              *(objects + (obje1 * throwobject) + (9)) = 0;

                            }	/* if firsttarget */
                        }	/* if all objects are set */
                    }		/* normal throw */

                  else
                    {		/* empty hand TODO: Error with multiplex zeroes */
                      (*(pattern[pos] + (phand * patt1) + (0))) = 1;	/* one NO object here */
                      (*(pattern[pos] + (phand * patt1) + (1))) = -1;	/* a NO object = -1 */
                    }
                }		/* multiplexloop */

//                              if(options.debug){
              //                                      printf("was %d wieviel %d ",*(pattern[pos]+(phand*patt1)+(1))+1,
              //                                              *(pattern[pos]+(phand*patt1)+(0)) );
              //                              }


            }			/* people */
        }			/* hand */

      /* get memory for next line */

      pattern[pos + MAXTHROW] = (int *) calloc (sizeof (int), options.people * 2 * (options.mplexmax + 1));
      if (pattern[pos + MAXTHROW] == NULL)
        {
          printf ("Error: Not enough memory for pattern 3");
          return 1;
        }

      /* test if ready */

      allperiods = TRUE;
      for (count1 = 0; count1 < options.objects; count1++)
        {
          if (!(*(objects + (obje1 * count1) + (7))))
            allperiods = FALSE;
        }
      if (allperiods & (period1 == 2))
        {
          for (count1 = 0; count1 < options.objects; count1++)
            {
              period1 = kgv (period1, *(objects + (obje1 * count1) + (7)));
            }
        }

      if ((pos >= period1 / 2 + options.latestthrow + 1) & (period1 != 2))
        exit = TRUE;

      pos++;
      if (pos == (MAXOUTLINES - MAXTHROW - 1))
        {
          printf ("Error: Periodlength of is pattern more than maximum (%d)\n", MAXOUTLINES - MAXTHROW);
          return 2;
        }

    }
  while (!exit);		/* outer loop */

  options.swapperiod = period1;

  return 0;			/* everything is fine */

}				/* makepattern() */

/*E*/
