/*A*//* Head  */

/****h*  output.c *********
 *
 *	NAME
 *   output.c
 *
 *	FUNCTION
 *    this part generates the output to the file
 *    
 *    for changing the output, you have only to
 *    change this part.
*
*	CREATION DATE
*	  4-Mar-1997
*
*  HISTORY
*    040397 disconected this part from main.c
*    150397 first working version
*    170397 added beginning and ending (prologue & epilogue)
*		 070504 compute_dt() for realistic heights (WR)
*
*  TODO
*
*  BUGS 
*    the generation of beginnings and endings is buggy
*
*  BEAST METHODS
*    head()
*    writepattern()
*
              *******
*/

/* {{{ */
/* for folding recognition */
/* }}} */

/*E*/

/*-----------------------------------------*/
/*A*//*  External routines and Structures */

#include "define.h"

/*E*/
/*-----------------------------------------*/
/*A*//*  Global variables */

extern struct OPTIONS options;	/*  for some options */

//extern int *siteswap;                                                 /* external pointer for the decoded inputstring */
extern int site1, site2;
//extern int **pattern;                                                 /* external pointer to the generated pattern */
//extern int patt1;
extern int *objects;		/* used in makepattern.c */
extern int obje1;
extern int hdelay[];

extern throwspec_type *throwspec;
enum {OBJ_CLUB = 1, OBJ_BALL, OBJ_RING};

FILE *fout = NULL;		/* output Filepointer */

/*E*/
/*-----------------------------------------*/
/*A*/ int head (int argc, char *argv[], char *ssstring) {

/****** output.c/head *********
 *
 *	NAME
 *	  head
 *
 *	FUNCTION
 *    opens the file and writes the head of the output file
 *
 *  SYNOPSIS
 *    stderror = head(int argc, char *argv[],char *ssstring)
 *
 *  RESULT
 *    0 if successful
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    writepattern()
 *
 *  CHANGES: 
 *   WR 11298:
 *     more realistic handpositions: 
 *     removed h= t= o=
 *     added B= for 1 or more jugglers
 *     removed upt
 *
 *   WR 26800: 
 *      -h: c|1, r|2, b|3
 *      use j instead of p for positioning
 *
 *******
 */

  int count;
  int period;			/* period with prologue */
  int object;			/* for different objectcolours */
  int ballcolours[] =
    {34, 36, 31, 35, 39, 33, 40, 41, 42, 43, 44, 45, 32, 37, 38};
  char throwtype[MAXPEOPLE*2];  // left1, right1, left2, right2, 
  DOUBLE dt;


  /* open file */

  if ((fout = fopen (options.outname, "wt")) == NULL)
    {
      printf ("Error: cannot create file %s \n", options.outname);
      return 2;
    }
  for (count = 0; count < options.people; count++)   // set default hand positions:
    {
      throwtype[count*2] = 'c';
      throwtype[count*2+1] = 'c';
    }
/** Handpositionen sollen besondere Form haben **/
  if (options.handm) {
    if ( ! options.throwspec) { // -h only, not: -H or -S
      for (count = 0; count < options.people * 2; count++) {
        switch (argv[options.handm][count + 2]) {
        case '1':		/* Normal */
        case 'c':
          throwtype[count] = 'c';
          break;
        case '2':		/* Rckwrtskaskade */
        case 'C':
        case 'r':
          throwtype[count] = 'r';
          break;
        case '3':		/* Backcross */
        case 'b':
          throwtype[count] = 'b';
          break;
        case 0:		/* Ende des Strings */
          printf ("Warning: Please enter the same number of values after -h as hands used\n");
          break;
        default:
          printf ("Warning: handoption %c is not supported\n", (char) argv[options.handm][count + 2]);
          break;
        }
      }
    } else if (options.throwspec == SS_HANDS) {    // -H
      for (count = 0; count < options.people * 2; count++) {
        throwtype[count] = throwspec[count].throw;
      }
    }
  }				/* spezielle Handpositionen */
    
/** write head **/

  if (options.infile)
    {				/* input by file */
      fprintf (fout, "! site-swap %s", ssstring);
      fprintf (fout, "%% converted with GNU " PROGNAME "\n");
    }
  else
    {
			if (options.multihandsiteswap > 2 ) {
				fprintf (fout, "! site-swap %s using %d hands -", 
					argv[options.sswap], options.multihandsiteswap);
				fprintf (fout, " converted with GNU " PROGNAME "\n");
			} else {
				fprintf (fout, "! site-swap %s", options.inputsiteswap);
				fprintf (fout, " converted with GNU " PROGNAME "\n");
			}
			fprintf (fout, "%% Call: ");
      for (count = 0; count < argc; count++)
        {			/* Aufruf als Kommentar sichern */
          fprintf (fout, "%s ", argv[count]);
        }
    }
  fprintf (fout, "\n");

  if (options.people==1) {
    fprintf(fout,"B=0.085,1.65,3.34,185.0\n");
  } else {
    fprintf(fout,"B=1.5,1.76,4.12,238.0\n");
  }

  dt = compute_dt();

  dt /= options.mult;
  period = options.swapperiod;
  if (options.beginwait != -1)
    {
      period += options.beginwait;	/* prologue */
      period += options.latestthrow;	/* epilogue */
    }

  if (options.mark)
    {				/* different colours */
      fprintf (fout, "O=%d ", options.objects);
      for (object = 0; object < options.objects; object++)
        {
          fprintf (fout, "1#%d ", ballcolours[*(objects + (obje1 * *(objects + (obje1 * object) + (6))) + (5))]);
        }
      fprintf (fout, "\n");
    }
  fprintf (fout, "dt=%1.3f a=%d", dt, options.objektnr);
  if (options.hold)
    fprintf (fout, " l=1");
  fprintf (fout, "\n");

  if (options.people == 1)
    {				/* Jongleurposition setzen fr solo */
      fprintf(fout,"j 0 0 0 0 %c %c\n",throwtype[0],throwtype[1]);
    }
  if (options.people == 2)
    {				/* jongleurpositionen fr 2 people Passing */
      fprintf(fout,"j 0 %f 0 0 %c %c\n",-options.radius,throwtype[0],throwtype[1]);
      fprintf(fout,"180 %f 0 0 %c %c\n",options.radius,throwtype[2],throwtype[3]);
    }
  if (options.people == 0)  // SCHEINT NIE ANGESPRUNGEN ZU WERDEN (WR)
    /*A*/
    {				/* jongleurpositionen fr 3 people Feed */
      fprintf(stderr,"s22j: Huch. Hier war ich noch nie...\n");
    }
  /*E*/
  if (options.people > 2)
    {				/* jongleurpositionen fr mehr als 3 people oder Dreieck */

      fprintf (fout, "j ");
      for (count = 0; count < options.people; count++)
        {
          DOUBLE x1, y1, phi, phi_b;
          phi = 720.0 / (DOUBLE) (options.people * 2) * (DOUBLE) count;
          phi_b = phi * PI / 180.0;
          x1 = options.radius * cos (phi_b);
          y1 = options.radius * sin (phi_b);
          fprintf(fout,"%6.1f %6.2f %6.2f 0 %c %c\n",
                  phi-180.0,x1,y1,throwtype[count*2],throwtype[count*2+1]);
        }
    }
  return 0;
}				/* head() */

/*E*/
/*-----------------------------------------*/
/*A*/  // debris
#if 0
/** generate wrap around **/

if (options.wrap)
{
  fprintf (fout, "*\n");
  for (count = 0; count < options.objects; count++)
    {				/* von */
      
      wrap = output[(objekt[count][1]) + options.takte]
        [(objekt[count][0])]
        [1];			/*  ToDo Multiplex hier einbauen */
      
      fprintf (fout, "%c", z[wrap]);
    }
  fprintf (fout, "\n");
  
  for (count = 0; count < options.objects; count++)
    {				/* nach */
      fprintf (fout, "%c", z[count]);
    }
  fprintf (fout, "\n");
}

printf ("Ok\n");
return;
				/* writemuster */
#endif

/*E*/
/*-----------------------------------------*/
/*A*/ void writepattern () {

/*A*/ // header and identifiers

/****** output.c/writepattern *********
 *
 *	NAME
 *	  writepattern
 *
 *	FUNCTION
 *    writes the generated pattern to the file
 *
 *  SYNOPSIS
 *    writepattern()
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    head()
 *
 *  CHANGES
 *   WR 2jun2:
 *      added -H and -S
 *
 *******
 */

int count;			/* counter */
int pos;			/* counter for position */
int pos1;			/* long holdtime */
int mul;			/* counter for multiplying pattern */
int mult;			/* multiplyer for mul */
int mult1;			/* multiplyer for mul */
int person;			/* counter for people */
int hand;			/* counter for hands */
int mplex;			/* counter for multiplex */
int written;			/* amount of objects already written here */
int object;			/* object to write */
int object1;			/* for testing of 2 holds */
int object2;			/* for testing of 2 holds */
int space;			/* space between collumns */
int prologue;			/* start with no throw */
int epilogue;			/* ending until all balls are catched */

int changehands[MAXPEOPLE];	/* Vertauschen der Spalten */

char z[] =
  {'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'};

/*E*/

/*A*/ // init

 fprintf (fout, "*\n");

/** Identifizierungen vornehmen **/

 for (count = 0; count < options.people; count++) {				/* TODO : How to use this ... */
   if (hdelay[count * 2] > hdelay[count * 2 + 1])
     changehands[count] = 0;
   else
     changehands[count] = 0;
 }
 prologue = 0;
 epilogue = 0;
 if (options.beginwait != -1) {
   prologue = options.beginwait * -1;
   options.mplexmax = options.objects;	/* not perfect but works */
   epilogue = options.latestthrow;
 }

/*E*/
 
/*A*/ /** write pattern **/

 for (pos = prologue; pos < options.swapperiod + epilogue; pos++) {				/* position */
   for (mul = 0; mul < options.mult; mul++) {			/* multiply */
     mult = mul * 6 / options.mult;
     for (person = 0; person < options.people; person++) {			/* people */
       for (hand = 0; hand < 2; hand++) {		/* hands */
         mplex = 0;
         written = 0;
         do  {		/* multiplex */
           /* write one element */
           object = getobject (pos, mult, person, hand, mplex, changehands);

/*A*/  // -l           
           if ((options.hold) & (object == -1)) {
             /* If now this hand is empty (-) and the object before (object1) is
                the same as the object after (object2), then let this object be here, too. */
             object1 = getobject (pos - 1, mult, person, hand, mplex, changehands);
             object2 = getobject (pos + 1, mult, person, hand, mplex, changehands);
             if (object1 == object2) {
               object = object1;		/* hold the 2s instead of throwing */
             } else {
               /* Create longer hand holding times:
                  without -l: 
                  - 1
                  2 -
                  - 3
                  with -l:
                  - 1
                  2 1
                  2 -
                  2 3
                  - 3
               */
               mult1 = mult + 3;		/* change offset */
               pos1 = (pos - 1 + options.swapperiod) % options.swapperiod;	/* the previous throw */
               if (mult1 > 6) {
                 mult1 -= 6;
                 pos1--;
               }
               object1 = getobject (pos1, mult1, person, hand, mplex, changehands);
               if (object1 != -1)
                 object = object1;
             }
           }	/* hold */
/*E*/

           if (object == -1) {
             if ( ! written) {	/* if written then do nothing (exit) */
               fprintf (fout, "-");
             }
           } else {
             if (object != -2) {	
               if (written > 0)
                 fprintf (fout, "&");
               switch (options.throwspec) {
               case SS_HANDS:
//                 fprintf(fout,"#h%d#",person*2+hand);
                 writeKOT(person*2+hand, options.throwspec);
                 break;
               case SS_SITESWAP: 
//               height = (float) (*(siteswap + ((pos%options.length) * site1) + (person * site2) + (mul*3 + 1))) / 6.0;
//  fprintf(fout,"$%d$",pos1);
//                 fprintf(fout,"#s%d#",pos%options.length);
                 writeKOT(pos%options.length, options.throwspec);
                 break;
               }
               fprintf (fout, "%c", z[object]);
               written++;
             }
           }	
           /* end of write one element */
           mplex++;
         } // do

         while (object != -1);		/* multiplex */
         if (written == 0)
           written = 1;
         for (space = 0; space < options.mplexmax - written; space++) {
           fprintf (fout, "  ");	/* space between hands */
         }		
         fprintf (fout, " ");	/* minimum space between hands */
       }		/* Hands */
       if (!(person == (options.people - 1)))
         fprintf (fout, "  ");	/* space between columns */
     }			/* people */
     fprintf (fout, "\n");	/* end of one row */
   }			/* end of one throw for each person */
 }				/* pos */
/*E*/
 
 printf ("Ok\n");
 return;
}
/*E*/
/*-----------------------------------------*/
/*A*/void writeKOT(int idx, int mode) {

  /* write bounce and kind of throw into pattern */

  char mod[30];
  int b, t;

  t = throwspec[idx].bounce;
  if (t>0) 
    for (b=0;b<t;b++)
      fprintf(fout,"/");
  else if (t != SS_DEFAULT)
    for (b=0;b<-t;b++)
      fprintf(fout,"\\");
  
  if (throwspec[idx].spin != SS_DEFAULT) {
    sprintf(mod,"%c%d", 
            mode == SS_SITESWAP && throwspec[idx].throw != 'c' ? throwspec[idx].throw : '*',
            throwspec[idx].spin);
  } else if (mode == SS_SITESWAP && throwspec[idx].throw != 'c') {
    sprintf(mod,"%c*", throwspec[idx].throw);
  } else {
    mod[0]='\0';
  }
  if (throwspec[idx].Zrot) {
    char zrot[5];
    sprintf(zrot,"z%d",throwspec[idx].Zrot);
    strcat (mod, zrot);
  }

  if (strlen(mod)) 
    fprintf(fout, "(%s)", mod);
}

/*E*/
/*-----------------------------------------*/
/*A*/DOUBLE compute_dt () {

	/* compute time step (dt) which	depends on:
			- type of object
			- number of objects
			- number of jugglers
			- height of throws
			- period of site swap pattern
	
		11jun4: some optimizations to get the number of spins right
		19mar5: reduce dt for patterns like 62020330
	*/

	DOUBLE dt, dt_hi, dt_lo;
	int objtype = OBJ_CLUB;
	int interpol = 0;
	DOUBLE max, fl_max, ce_max;

	// type of objects:

  switch (options.objektnr) { 
		case 2:
		case 3:
		case 47:
		case 49:
		case 65: 
			objtype = OBJ_RING;
			break;
		default:
			if (options.objektnr == 6 || options.objektnr == 12 ||
				(options.objektnr >=31 && options.objektnr <=45) ||
				(options.objektnr >=50 && options.objektnr <=64)) {  
				objtype = OBJ_BALL;
			}
  }
	/* cascade, fountain, shower, half shower (basic solo patterns): len == 1 or 2
		passing: 333p3p (len == 2), 33333p3p (len == 3) */
	if (options.length <= 2) {
		// time step depends on highest throw
		max = options.maxheight;
	} else if (options.objects <=3) {
		// max is a bit lower than max no of objects -> juggling is slower 
		max = options.maxheight/1.25;
	} else {
		// time step depends on number of objects (site swaps are juggled slower than basic patterns)
		max = (float)options.objects / (float)options.people;
	}

	if ( (DOUBLE)((int)max) == max)  {
		dt = compute_dt_int_height(objtype,(int)max);   // height was integer
	} else {
		fl_max = floor(max);
		ce_max = ceil(max);
		dt_lo = compute_dt_int_height(objtype,(int)fl_max);
		dt_hi = compute_dt_int_height(objtype,(int)ce_max);
		// linear interpolation between these 2 values:
		dt = dt_lo + (max-fl_max)/(ce_max-fl_max)*(dt_hi-dt_lo);
		interpol = 1;
	}
	
	if (options.length <= 2 && options.people > 1 && interpol == 0 && options.all_throws_same_height == 0)
		dt *= 1.19;  // some passing patterns are slower than solo juggling

	return dt;
} 

/*E*/
/*-----------------------------------------*/
/*A*/DOUBLE compute_dt_int_height(int objtype, int max) {

	// return timestep based on integer throw height

	DOUBLE dt;

	switch (objtype) {  // stupid but readable code:
		case OBJ_BALL:
			switch (max) {
			case 1: 
			case 2:
			case 3: dt = .40; break;
			case 4: dt = .30; break;
			case 5: dt = .28; break;
			case 6: dt = .25; break;
			case 7: dt = .23; break;
			case 8: dt = .21; break;
			case 9: dt = .19; break;
			default: dt = .18; break;  // 10 and more
			break;
			}
			break;
		case OBJ_CLUB:
			switch (max) {
			case 1: 
			case 2:
			case 3: dt = .40; break;
			case 4: dt = .36; break;
			case 5: dt = .32; break;
			case 6: dt = .30; break;
			case 7: dt = .26; break;
			default: dt = .24; break;  // 8 and more
			break;
			}
			break;
		case OBJ_RING:
			switch (max) {
			case 1: 
			case 2:
			case 3:
			case 4: dt = .35; break;
			case 5: dt = .30; break;
			case 6: dt = .27; break;
			case 7: dt = .24; break;
			case 8: dt = .22; break;
			case 9: dt = .21; break;
			case 10: dt = .20; break;
			case 11: dt = .19; break;
			case 12: dt = .18; break;
			default: dt = .17; break;  // 13 and more
			}	
			break;
			// thanks to Garfield, Gatto and Lucas for samples of realistic juggling heights.
	}

	return dt;

} 

/*E*/
/*-----------------------------------------*/
