/*A*//* Head  */
/****h* precalc.c *********
 *
 *	NAME
 *	  Precalc.c
 *
 *	FUNCTION
 *    this part calculates the values for
 *    errorfree generation of the pattern.
*
*	CREATION DATE
*	  4-Mar-1997
*
*  HISTORY
*    150397 first working version
*    160397 calcperiod removed
*    160397 cross and cross passing works
*    200397 detection of wrong handdelaytimes
*    220397 fixed syncpassing bug
*    201104 WR added code: -H & -S work together with -m (partly...)
*
*  TODO
*
*  BUGS 
*
*  BEAST METHODS
*
*    identifystring()
*    parsestring()
*    setthrow()
*    handdelay()
*
*******
*/
/* {{{ */
/* for folding recognition */
/* }}} */

/*E*/
/*----------------------------------------*/
/*A*//*  External routines and Structures */

#include "define.h"

/*E*/
/*----------------------------------------*/
/*A*//*  Global variables */

extern struct OPTIONS options;	/* for some options */
extern int *siteswap;		/* external pointer for the decoded inputstring */
extern int site1;
extern int site2;
/*
  small global arrays (not dynamically allocated)
*/

int hdelay[MAXPEOPLE * 2];	/* Table for handdelay */

throwspec_type *throwspec;

/*E*/
/*----------------------------------------*/
/*A*/ int handdelay ()

{
/****** precalc.c/handdelay *********
 *
 *	NAME
 *	  handdelay
 *
 *	FUNCTION
 *
 *    Now the moment for each hand throwing a object will be defined
 *    and stored in the hdelay array.
 *    i.e for 5 ball halfshower which is 5.5 4.5 => L=0 R=1/2
 *    That is also a basis for the pattern stretching factor in
 *    the output.c part
 *
 *  SYNOPSIS
 *    stderror = handdelay()
 *
 *  RESULT
 *    0 if successful else stderror
 *
 *  BUGS
 *
 *  TODO
 *    Error checking
 *
 *  SEE ALSO
 *
 *******
 */

  int count;			/* Counter */
  int throw;			/* throw  */
  int mult;			/* Counter2  */
  int count3;			/* Counter3  */
  int mul;			/* factor for pattern stretching */
  int hands;			/* number of hands */

  int target;			/* target of throw */
  int targetpoint;		/* targettime of throw */
  int targetperson;		/* person to whom the pass goes */
  int targethand;		/* hand which is target for a pass */
  int height;			/* height of throw */
  int sourcethrow;		/* throw in the siteswap */
  int changehands;		/* ofset of change for hands */
  int wantthrow;		/* 0 for straight 1=cross desired throwmode */
  int possiblethrow;		/* 0 for none 1=straight 2=cross 3=both */
  int hdelaychanged[MAXPEOPLE * 2];	/* stores if hands have already been changed if yes -> error */

  mul = 1;			/* Define stretcing as one (standard) */

  if (options.hold)
    mul = 2;			/* if longer handholdtimes, then we have to stretch the pattern */

  hands = (options.people) * 2;

  for (count = 0; count < hands; count++)
    {
      hdelay[count] = (count % 2) * 6;	/* Normal      :0101 */
      hdelaychanged[count] = 0;
    }
  if (options.sync)
    for (count = 0; count < hands; count++)
      hdelay[count] = ((count % 4) / 2) * 6;	/* synchronous :0011 */

  if (options.syncpass)
    for (count = 0; count < hands; count++)
      hdelay[count] = 0;	/* syncpassing :0000 */

  if (options.oddcross)
    for (count = 0; count < hands; count++)
      hdelay[count] = ((count % 4) / 2) * 6;	/* oddcross    :0110 */

  /* TODO: check for exclusive or */

/** Now check only the selfs for fractional throws **/

  for (count = 0; count < options.people; count++)
    {
      for (throw = 0; throw < kgv (options.length, 2); throw++)
        {			/* double patternlenth if odd */

          sourcethrow = throw % options.length;
          for (mult = 0; mult < (*(siteswap
                                   + sourcethrow * site1
                                   + count * site2)) * 3; mult += 3)
            {			/* go through the multiplex throws */

              target = *(siteswap + (sourcethrow * site1)
                         + (count * site2)
                         + (mult + 2));		/* target for throw */

//printf ("target %d \n",target);

              if (target < 2)
                {		/* examine just the selfs */

                  height = *(siteswap + (sourcethrow * site1)
                             + (count * site2)
                             + (mult + 1));	/* height of throw */

//printf ("height %d \n",height);

                  if ((target == 1) && (height % 12 != 0))
                    {		/* forced self? */
                      printf ("Warning: only an even throw can be forced to cross (X)\n");
                    }

                  if ((target == 1) && (height % 12 == 0))
                    {

/** forced cross **/
                      /* just checking fs this is ok and change hand if not */

                      targetpoint = (height + hdelay[throw % 2 + count * 2]) % 12;	/* target point in time */

//printf ("targetp %d \n",targetpoint);

                      if (hdelay[(throw + 1) % 2 + count * 2] != targetpoint)
                        {
                          if (hdelaychanged[(throw + 1) % 2 + count * 2] == 1)
                            {
                              printf ("Error: The self handdelaytimes are not solvable\n");
                              return 2;
                            }
                          if (options.debug)
                            {
                              printf ("changing hand %d of person %d due to the throw %2.1f (no.) %d from %2.1f to %2.1f (to synchron mode)\n",
                                      (throw + 1) % 2,
                                      count,
                                      (float) height / 6,
                                      throw,
                                      (float) (hdelay[(throw + 1) % 2 + count * 2]) / 6,
                                      (float) targetpoint / 6);
                            }
                          hdelay[(throw + 1) % 2 + count * 2] = targetpoint;
                          hdelaychanged[(throw + 1) % 2 + count * 2] = 1;
                        }
                      *(siteswap + (sourcethrow * site1)
                        + (count * site2)
                        + (mult + 3)) = 1;	/* absolute target is a cross */
                    }
                  /* TODO: Check this\/ */
                  if ( /*(target==0)&& */ (height % 12 != 0))
                    {

/** no collumn, but not forced cross **/

                      *(siteswap + (sourcethrow * site1)
                        + (count * site2)
                        + (mult + 3)) = 1;	/* target is the other hand */

                      targetpoint = (height + hdelay[throw % 2 + count * 2]) % 12;	/* target point in time */

//printf ("targetp %d \n",targetpoint);

                      if (hdelay[(throw + 1) % 2 + count * 2] != targetpoint)
                        {
                          if (hdelaychanged[(throw + 1) % 2 + count * 2] == 1)
                            {
                              printf ("Error: The self handdelaytimes are not solvable\n");
                              return 2;
                            }
                          if (options.debug)
                            {
                              printf ("changing hand %d of person %d due to the throw %2.1f (no.) %d from %2.1f to %2.1f\n",
                                      (throw + 1) % 2,
                                      count,
                                      (float) height / 6,
                                      throw,
                                      (float) (hdelay[(throw + 1) % 2 + count * 2]) / 6,
                                      (float) targetpoint / 6);
                            }
                          hdelay[(throw + 1) % 2 + count * 2] = targetpoint;
                          hdelaychanged[(throw + 1) % 2 + count * 2] = 1;
                        }
                    }
/** we don't have to care for collumns, because they won't go to another hand **/

                }		/* if */

            }			/* for mult */
        }			/* for throw */
    }				/* for count */


/** Now check the passes for fractional and sync throws **/

  for (count = 0; count < options.people; count++)
    {
      for (throw = 0; throw < options.length * 2; throw++)
        {			/* double the patternlenth for check */

          sourcethrow = throw % options.length;
          for (mult = 0; mult < (*(siteswap
                                   + sourcethrow * site1
                                   + count * site2)) * 3; mult += 3)
            {			/* go through the multiplex throws maybe wrong */

              target = *(siteswap + (sourcethrow * site1)
                         + (count * site2)
                         + (mult + 2));		/* target for throw */

              if (target > 1)
                {		/* examine just the passes */

/**      check routine     **/

                  height = *(siteswap + (sourcethrow * site1)
                             + (count * site2)
                             + (mult + 1));	/* height of throw */

//printf ("Ptarget %d \n",target);
                  //printf ("Pheight %d \n",height);

                  /* set passing target */

                  if ((target == 2) | (target == 3))
                    targetperson = (count + 1) % options.people;	/* next person */
                  else
                    targetperson = (target - 4) / 2;	/* defined person (for 3 and more) */

                  targetpoint = (height + hdelay[throw % 2 + count * 2]) % 12;	/* target point in time */

//printf ("targetpers %d \n",targetperson);
                  //printf ("Ptargetp %d \n",targetpoint);

/** set desired throwmode **/

                  if ((((height / 6) % 2 == 1) & (target % 2 == 0)) |
                      (((height / 6) % 2 == 0) & (target % 2 == 1)))
                    {
                      wantthrow = 0;	/* straight */
                    }
                  else
                    {
                      wantthrow = 1;	/* cross */
                    }
                  if (height % 6 != 0)
                    {		/* fractional passes, throw where possible */
                      wantthrow += 2;
                    }
/*** check possible throwtargets ***/

                  possiblethrow = 0;
                  if (hdelay[(throw + 1) % 2 + targetperson * 2] == targetpoint)
                    {
                      possiblethrow += 1;
                    }
                  if (hdelay[(throw) % 2 + targetperson * 2] == targetpoint)
                    {
                      possiblethrow += 2;
                    }

                  switch (possiblethrow)
                    {

                    case 0:	/* no throw possible */

                      /* none of the hands to which the pass goes is correct */

                      targethand = (throw + (height / 6)) % 2;	/* change hand when 3,5,7... */
                      changehands = targetpoint - hdelay[targethand + targetperson * 2];
//printf ("Change %d \n",changehands);
                      if (hdelaychanged[(throw + 1) % 2 + targetperson * 2] == 2)
                        {
                          printf ("Error: The passing handdelaytimes are not solvable\n");
                          return 2;
                        }
                      if (options.debug)
                        {
                          printf ("changing hands of person %d due to the pass no. %d (",
                                  targetperson,
                                  throw + 1);
                          printthrow (throw / 2, count, throw % 2, mult);
                          printf (")\nfrom %2.1f to %2.1f (left) and from %2.1f to %2.1f (right)\n",
                                  (float) ((hdelay[targetperson * 2])) / 6,
                                  (float) ((hdelay[targetperson * 2]) + changehands) / 6,
                                  (float) ((hdelay[1 + targetperson * 2])) / 6,
                                  (float) ((hdelay[1 + targetperson * 2]) + changehands) / 6);
                        }	/* if debug */
                      hdelay[targetperson * 2] = hdelay[targetperson * 2] + changehands;
                      hdelay[1 + targetperson * 2] = hdelay[1 + targetperson * 2] + changehands;
                      hdelaychanged[(throw + 1) % 2 + count * 2] = 2;

                      *(siteswap + (sourcethrow * site1)
                        + (count * site2)
                        + (mult + 3)) = targetperson * 2 + 2 + (height / 6) % 2 + target % 2;	/* set absolute target (see below) */
// printf("new target: %d\n",targetperson*2+2+(height/6));


                      break;
                    case 1:	/* straight throw possible */
                      if ((wantthrow % 2 == 0) | (wantthrow > 1))
                        {

                          *(siteswap + (sourcethrow * site1)
                            + (count * site2)
                            + (mult + 3)) = targetperson * 2 + 3;	/* set absolute target straight */

                        }
                      else
                        {	/* exchange the hands */

                          if (options.debug)
                            {
                              printf ("exchanging hands of person %d due to the pass no. %d (", targetperson, throw + 1);
                              printthrow (throw / 2, count, throw % 2, mult);
                              printf (") s\n");
                            }	/* if debug */
                          changehands = hdelay[targetperson * 2];
                          hdelay[targetperson * 2] = hdelay[1 + targetperson * 2];
                          hdelay[1 + targetperson * 2] = changehands;
                          hdelaychanged[(throw + 1) % 2 + count * 2] = 2;
                        }
                      break;
                    case 2:	/* cross throw possible */
                      if ((wantthrow % 2 == 1) | (wantthrow > 1))
                        {

                          *(siteswap + (sourcethrow * site1)
                            + (count * site2)
                            + (mult + 3)) = targetperson * 2 + 2;	/* set absolute target cross */

                        }
                      else
                        {	/* exchange the hands */

                          if (options.debug)
                            {
                              printf ("exchanging hands of person %d due to the pass no. %d (", targetperson, throw + 1);
                              printthrow (throw / 2, count, throw % 2, mult);
                              printf (") x\n");
                            }	/* if debug */
                          changehands = hdelay[targetperson * 2];
                          hdelay[targetperson * 2] = hdelay[1 + targetperson * 2];
                          hdelay[1 + targetperson * 2] = changehands;
                          hdelaychanged[(throw + 1) % 2 + count * 2] = 2;
                        }
                      break;

                    case 3:	/* both throws are possible */
                      *(siteswap + (sourcethrow * site1)
                        + (count * site2)
                        + (mult + 3)) = targetperson * 2 + 2 + (wantthrow + 1) % 2;	/* set absolute target to wantthrow */
                      break;

                    default:
                      printf ("Error: Internal Error at Handdelay switch possiblethrow\n");
                      return 2;
                    }
                }		/* if from passes check routine */
            }			/* for mult */
        }			/* for throw */
    }				/* for count */

  /* debug output */
  if (options.debug)
    {
      printf ("\nChanged targets:\n\n");
      for (count3 = 0; count3 < options.people; count3++)
        {
          printf ("Person Number: %d\n", count3 + 1);
          for (count = 0; count < options.length; count++)
            {
              printf ("Nr. %d objects: %d ", count,
                      *(siteswap + (count * site1) + (count3 * site2) + (0)));
              for (mult = 0; mult < (*(siteswap + (count * site1) + (count3 * site2))) * 3; mult += 3)
                {
                  printf ("Height: %2.1f Target: %d ",
                          (float) (*(siteswap + (count * site1) + (count3 * site2) + (mult + 1))) / 6,
                          *(siteswap + (count * site1) + (count3 * site2) + (mult + 3)));	/* changed target */
                }
              printf ("\n");
            }
        }
    }				/* debug output */
/*

Now the targets of the throws are changed to:

Selfs:

0=column
1=cross self (with forced cross)

Passes:

2=pass to person 0 cross    (does not change hands i.e from Left to Left )
3=pass to person 0 straight (does change hands)
4=like 2 for person 1
5=like 3 for person 1
6=like 2 for person 2 .. and so on

*/

  for (count = 0; count < hands; count++)
    {
      switch (hdelay[count])
        {

        case 0:		/* standard */
        case 6:
          break;
        case 2:		/* 1/3 delay */
        case 4:
        case 8:
        case 10:
          if (mul < 2)
            mul = 3;
          if (mul == 2)
            mul = 6;
          break;
        case 3:		/* 1/2 delay */
        case 9:
          if (mul < 2)
            mul = 2;
          if (mul == 3)
            mul = 6;
          break;
        default:
          fprintf (stderr, "Internal error: hdelay1");
          break;
        }
    }				/* for */
  options.mult = mul;

      printf ("Patternstretchfactor: %d\n", mul);


  if (options.debug)
    {
      printf ("Patternstretchfactor: %d\n", mul);
      printf ("Handdelays: ");
      for (count = 0; count < hands; count++)
        printf ("%1.1f ", (float) hdelay[count] / 6);
      printf ("\n");
    }				/* debug output */
  return 0;

}				/* handdelay() */

/*E*/
/*----------------------------------------*/
/*A*/ int identifystring (char *argv[], char *siteswap)
{
/****** precalc.c/identifystring *********
 *
 *	NAME
 *	  indentifystring
 *
 *	FUNCTION
 *    This function identfies the class of the site swap and sets
 *    in the options structure:
 *
 *    mplex    - Multiplex
 *    people   - Number of People for passing (1=solo Juggling)
 *    sync     - Synchronous Juggling
 *    objectnr - number of default object (club for passing else ball)
 *    fract    - fractional throws (not used)
 *
 *  SYNOPSIS
 *    result = identifystring( char *argv[],char *siteswap)
 *
 *  RESULT
 *    0 if successful
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    parsestring()
 *
 *******
 */

  char *m;			/* input String */
  int swaplength = 0;		/* length of input string */

	if ( strchr (options.inputsiteswap, '*') ) {
		// perform star operation: 6x4* --> 6x4 46x
	  char ss_temp[MAXINPUTLEN];
		strncpy (ss_temp, options.inputsiteswap, MAXINPUTLEN);
		star_convert(ss_temp, options.inputsiteswap, options.solo_mode);
	}

  m = options.inputsiteswap;	/* krzerer Name */
  if (options.infile)
    {
      m = siteswap;
    }
  
  while (*m)
    {
      switch (*m)
        {
        case 'O':
        case 'N':
        case 'M':
        case 'L':
        case 'K':
          printf ("Error: More than 10 people are not supported");
          return 2;
          break;
        case 'J':
          if (options.people < 10)
            options.people = 10;
          break;
        case 'I':
          if (options.people < 9)
            options.people = 9;
          break;
        case 'H':
          if (options.people < 8)
            options.people = 8;
          break;
        case 'G':
          if (options.people < 7)
            options.people = 7;
          break;
        case 'F':
          if (options.people < 6)
            options.people = 6;
          break;
        case 'E':
          if (options.people < 5)
            options.people = 5;
          break;
        case 'D':
          if (options.people < 4)
            options.people = 4;
          break;
        case 'C':
          if (options.people < 3)
            options.people = 3;
          break;
        case 'B':
        case 'p':
        case 'P':
          if (options.solo_mode == 0 && options.people < 2)
            options.people = 2;
          break;
        case 'x':
        case 'X':
          if (!options.verz)
            options.sync = TRUE;
          break;
        case '[':
        case ']':
          if (!options.verz)
            options.mplex = TRUE;
          break;
//        case ',':   // TODO: stimmt das? (4,4x) hat auch ein , drin.
        case '.':
          if (!options.fract)
            options.fract = TRUE;
          break;
        }
      swaplength++;
      m++;
    }				/* end of Identification */
  
  /* swaplength is len(argv[]) and not the period of the siteswap pattern */
  options.swaplength = swaplength;

  /* set standard objects according to number of people */
  
  if ( ! (options.objektnr)) {
    if (options.people < 2) {
      options.objektnr = 6;	/* Balls  */
    } else {
      options.objektnr = 5;	/* Clubs  */
    }
  }
  return 0;
}				/* identifystring */
/*E*/
/*----------------------------------------*/
/*A*/ int parsestring (char *argv[], char *ssstring)
{
/****** precalc.c/parsestring *********
 *
 *	NAME
 *	  parsestring
 *
 *	FUNCTION
 *
 *    This function fills the siteswap array with the
 *    sitewap string
 *    
 *  SYNOPSIS
 *    stderror = parsestring( char *argv[],char *ssstring)
 *
 *  RESULT
 *    0 if successful else stderror
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    identifystring()
 *
 *******
 */
  char *m;			/* input String */

  int totallength;		/* Length * People  */
  int count;			/* Schleifenvariable   */
  int count2;
  int count3;
  int sum = 0;			/* sum der Wurfhhen */
  int objekte;			/* Anzahl der Objekte  */
  int haende;			/* people *2         */
  int mplex;			/* multiplex is on       */
  int height;			/* gelesene Wurfhhe   */
  int person;			/* welche Hand gerade dran ist */
  int ziel;			/* ziel eines Passes   */
  int mplexmax;			/* maximum multiplex throws */
  char zeichen;			/* zu untersuchendes Zeichen */
  DOUBLE ddreh;			/* durchschnittliche Drehung */
  DOUBLE freq;			/* frequenz */
  DOUBLE dji;			/* Dancey's juggling index from the encyclopedia of ball juggling */
  DOUBLE ssd = 0;		/* siteswap difficulty */

/*A*/ /* Get Memory for siteswap */

  siteswap = (int *) calloc (sizeof (int), 
                             options.swaplength * options.people * (MAXMULTIPLEX * 3 + 1));
  site1 = options.people * (MAXMULTIPLEX * 3 + 1);
  site2 = MAXMULTIPLEX * 3 + 1;
  if (siteswap == NULL)
    {
      printf ("Error: Not enough memory for siteswap");
      return 1;
    }
  /*E*/

  m = options.inputsiteswap;	/* get the stringpointer */
  if (options.infile) {
    m = ssstring;
  }
  options.length = 0;
  options.mplexmax = 0;
 	options.maxheight = 0.0;
	mplexmax = 0;
  mplex = 0;
  height = -1;
  person = 0;

  while (*m) {
    zeichen = *m;
    switch (zeichen)  {
    case 10:
    case 13:
      zeichen = 32;
      break;
    case 32:
    case '(':
    case ')':
    case '<':
    case '{':
    case '>':
    case '}':
    case '|':		/* just ignore this */
    case ',':		// comma within sync descriptions.
//          fprintf(stderr,"s22j: ignored: %c\n",zeichen);
      break;
    case '.':		/* point means special heights, don't use comma here! */
      m++;
      zeichen = *m;		/* get next char */
      switch (zeichen) {
      case '0':		/* cross  (wird im Moment ignoriert */
        break;
      case '3':		/* 1/3 Takt */
        height += 2;	/* 2/6= 1/3 */
        break;
      case '5':		/* 1/2 Takt */
        height += 3;
        break;
      case '6':
      case '7':		/* 2/3 Takt */
        height += 4;
        break;
      default:
        printf ("Error: Character %c after decimal point is invalid\n"
                "please use only 3, 5 or 7\n", zeichen);
        break;
      }
      break;
    case '[':
      setthrow (&person, &height, &sum, 0, mplex);
      mplex = 1;		//Multiplex an
      break;
    case ']':
      setthrow (&person, &height, &sum, 0, mplex);
      mplex = 0;
      if (mplexmax < options.mplexmax)  {
        mplexmax = options.mplexmax;
      }
      person++;
      if (person == options.people)    {
        person = 0;
        options.length++;
      }
      break;
    case 'x':
    case 'X':
      ziel = 1;
      if ((*(m + 1) == 'P') | (*(m + 1) == 'p'))        {			/* cross Pass */
        ziel = 3;
        m++;
      }
      setthrow (&person, &height, &sum, ziel, mplex);
      options.sync = 1;
      break;
    case 'J':
    case 'I':
    case 'H':
    case 'G':
    case 'F':
    case 'E':
    case 'D':
    case 'C':
    case 'B':
    case 'A':
    case 'P':
    case 'p':
      if ( ! (*m == 'p' && options.solo_mode == 1) )  {
				// skip this if solo mode and throw was 'p'
				ziel = zeichen -= 'A';
				if ((ziel == 'P' - 'A') | (ziel == 'p' - 'A'))        {
					ziel = 2;
				}      else            {
					ziel = ziel * 2 + 4;
				}
				if ((*(m + 1) == 'X') | (*(m + 1) == 'x'))        {			/* cross Pass */
					ziel++;
					m++;
				}
				setthrow (&person, &height, &sum, ziel, mplex);
				break; 
			}

	 default:

      setthrow (&person, &height, &sum, 0, mplex);

      if (zeichen >= 'a' && zeichen <= 'o') {			// von a-o = Hhe 10-30
        zeichen -= 'a' - 10;	// Zahlen ab 10 werden mit Buchstaben dargestellt
      } else if (zeichen >= '0' && zeichen <= '9')  {
        zeichen -= '0';	// Zahlen unter 10 werden mit Ziffern dargestellt!
      } else if (zeichen >= 'p' && zeichen <= 'z' && options.solo_mode == 1) {
				zeichen -= 'a'-10;  // solo mode: throws from 'p' thru 'z' are allowed. 
			} else {
        printf ("s22j: Illegal character '%c' (0x%x) in pattern definition.\n", zeichen, zeichen);
        return 1;
      }
      height = zeichen * 6;	/* Wurfhhe merken (6 unterteilungen = 1 Takt) */
			break;
    }			/* switch */
		if (options.maxheight < (double)height/6.0)
			options.maxheight = (double)height/6.0;  // save highest throw
    m++;
  }				/* while */
  
  setthrow (&person, &height, &sum, 0, mplex);
  
  if ( ! options.length)
    return 2;			/* error: no inputsiteswap */
  if ( ! sum)
    return 2;			/* error:  0 inputsiteswap */
  
  if (sum % 6) {				/* gehen wurfhhen auf? */
    fprintf (stderr, "Warning: the throws should add to an integer value\n");
  }
  if (person) {
    printf ("Error: The input must have n*people elements\n");
    return 2;
  }
  sum /= 6;
  
  totallength = options.length * options.people;
  objekte = sum / options.length;
	if ( (float)sum / (float)totallength == options.maxheight)
		options.all_throws_same_height = 1;
	else
		options.all_throws_same_height = 0;
  haende = options.people * 2;
  ddreh = (sum - 2 * totallength) / totallength;
  freq = 2.0 * (objekte - haende) / haende / ddreh;
  options.objects = objekte;
  if ( ! mplexmax)
    mplexmax = 1;
  options.mplexmax = mplexmax;
  
  if (sum % options.length) {
    printf ("Error: Could not calculate the number of objects\n");
    return 3;
  }
  printf ("%d Objects, %d Hands, Period %d, Frequency %3.3f, Average rotation %3.3f.\n", 
          objekte, haende, options.length, freq, ddreh);
  
  for (count3 = 0; count3 < options.people; count3++)  {
    for (count = 0; count < options.length; count++) {
      for (count2 = 0; count2 < (*(siteswap + (count * site1) + (count3 * site2))) * 3; count2 += 3)  {
        ssd += abs ((float) (*(siteswap + (count * site1) + (count3 * site2) + (count2 + 1))) / 6 - objekte / options.people);
      }
    }
  }
  /* calculate dancey's juggling index from the EBJ */
  dji = ((DOUBLE) objekte) / ((DOUBLE) (haende + ((DOUBLE) haende) / objekte));
  ssd /= (DOUBLE) totallength;
  printf ("Dancey's Juggling Index: %2.3f Siteswap Difficulty: %2.3f\n", dji, ssd);
  
  if (options.debug) {
    for (count3 = 0; count3 < options.people; count3++) {
      printf ("Person number: %d\n", count3 + 1);
      for (count = 0; count < options.length; count++) {
        printf ("Nr. %d Objects: %d ", count,
                *(siteswap + (count * site1) + (count3 * site2) + (0)));
        for (count2 = 0; count2 < (*(siteswap + (count * site1) + (count3 * site2))) * 3; count2 += 3)  {
          printf ("Height: %2.1f Target: %d ",
                  (float) (*(siteswap + (count * site1) + (count3 * site2) + (count2 + 1))) / 6,
                  *(siteswap + (count * site1) + (count3 * site2) + (count2 + 2)));
        }
          printf ("\n");
      }
    }
  }				/* debug output */
  return 0;
  
  
}				/*parsestring() */
/*E*/
/*----------------------------------------*/
/*A*/ void setthrow (int *who, int *height, int *sum, int target, int mplex)
{
/***** precalc.c/setthrow *********
 *
 *	NAME
 *	  setthrow
 *
 *	FUNCTION
 *
 *    This function saves the throw in the sitewap buffer
 *    This is a subroutine of parsestring
 *
 *  SYNOPSIS
 *    setthrow(int *who,int *height,int *sum,int target,int mplex)
 *
 *  RESULT
 *    none
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    parsestring()
 *
 *******
 */

  int where;

  if (*height == -1)
    return;			/* No value to set */

//printf("height %d",*height);
  (*(siteswap + (options.length * site1) + (*who) * site2))++;		/* one more throw here */
  where = ((*(siteswap + (options.length * site1) + (*who) * site2)) - 1) * 3;		/* which throw? */
  (*(siteswap + (options.length * site1) + (*who) * site2 + where + 1)) = *height;	/* save height */
  (*(siteswap + (options.length * site1) + (*who) * site2 + where + 2)) = target;	/* save target */
  (*(siteswap + (options.length * site1) + (*who) * site2 + where + 3)) = 0;	/* init 2.target */
  if (!mplex)
    {
      (*who)++;
      if (*who == options.people)
        {
          *who = 0;
          (options.length)++;
        }
    }
  else
    {
      (options.mplexmax)++;
    }
  *sum += *height;
  *height = -1;			/* clear for next throw */
  return;

}				/*settrow() */
/*E*/
/*----------------------------------------*/
/*A*/ int throw_spec (char *argv[]) {

  /* read options from -H and -S 
   *
   * return 0 on success 
   */

  throwspec_type *thsp;
  int p, len;
  char *argline, *al, sptemp[10], botemp[10];

  switch (options.throwspec) {
  case SS_HANDS:
    options.thsp_max = options.people*2;
    break;
  case SS_SITESWAP:
    options.thsp_max = options.people * options.length;
    break;
  default:
    return 0;
  }
  
  len = strlen(argv[options.handm]) -2;  // contains -S or -H -> len is 2 longer than thsp
	if (options.multihandsiteswap > 2 && 
		options.throwspec == SS_SITESWAP) {  // multi handed siteswap 
		len *= options.multihandsiteswap/2;  // see convertMHSS
	}
  if ( len != LEN1HAND*options.thsp_max ) {
    printf("Wrong length of <throwspec> with %s: length is %d but should be %d\n",
           options.throwspec == SS_HANDS ? "-H" : "-S",
           len, LEN1HAND*options.thsp_max);
    return 3;
  }

  throwspec = calloc (options.thsp_max, sizeof(throwspec_type));
  if ( ! throwspec) {
    printf("No mem for throwspec!\n");
    return 1;
  }

  argline = malloc(len+2);  // +2 for newline and/or null byte
  if ( ! argline) {
    printf ("No mem for argline!\n");
    free (argline);
    return 2;
  }

  strcpy (argline, &argv[options.handm][2]);

  al = argline;

  for (p=0; p<options.thsp_max; p++, al+=LEN1HAND) {
    thsp = &throwspec[p];
    sscanf(al,"%1csp%2cbo%2cZ%3d",
           &thsp->throw, sptemp, botemp, &thsp->Zrot);
    sptemp[2] = '\0';  // make string 2 chars long
    botemp[2] = '\0';
		if ( ! strcmp(sptemp,"**"))
      thsp->spin = SS_DEFAULT;
    else
      thsp->spin = atoi(sptemp);
    if ( ! strcmp(botemp,"**"))
      thsp->bounce = SS_DEFAULT;
    else
      thsp->bounce = atoi(botemp);

/*
    printf("thsp[%d]: throw=%c  spin=%d (%s)  bounce=%d (%s)  Zrot=%d\n",
           p,thsp->throw, thsp->spin, sptemp, thsp->bounce, botemp, thsp->Zrot);
*/
  }
  
  free(argline);
  // free throwspec in exitprog()

  return 0;

}
/*E*/
/*----------------------------------------*/
